{
  Copyright 2011. Bruce Drummond.

  You may use and modify this software without any restriction.

  Please note that the following settings are for a GeckoDrive
  G251X microstepping driver in combination with an Applied
  Motion HT23-400 high torque stepper motor.  You may have to
  alter these settings to suit your particular driver, motor,
  and needs.

  The main settings for affecting motor operation are:
  
    START_STOP_SPEED

      and

    RAMP

  By altering these two settings, a person can achieve many
  different ramping profiles.  So go ahead and experiment a
  little and have some fun.   

  With a little creativitiy, this software can be modified to
  run for a specified number of steps and with a specified
  direction of rotation.

  If you create a full blown stepper driver from this sample
  code, it is recommended to set up constants for all your pins
  and settings.  Additionally, if you create some useful code
  with this sample, please share it in the forum or OBEX.  
}

CON

  'Multiply the external frequency by 16.
  _CLKMODE              = XTAL1 + PLL16X

  'External crystal of 5MHz.
  _XINFREQ              = 5_000_000

  'Establish the clock frequency so that a couple constants may
  'be declared in the CON block.
  CLK_FREQ              = ((_CLKMODE - XTAL1) >> 6) * _XINFREQ

  'The GeckoDrive G251X microstepping driver requires a minimal
  'pulse width of 1us, so establish this setting.
  HIGH_PULSE_WIDTH      = CLK_FREQ / 1_000_000

  'The code needs a certain amount of time to operate, so set
  'this.  Additonally, this setting can be used to control the
  'maximum speed of the motor.  Just remember that the maximum
  'execution speed takes priority over maximum speed of the
  'motor, otherwise the code will falter.
  MAX_EXE_MOTOR_SPEED   = CLK_FREQ / 20_000

  'Set the pin number for the pushbutton
  PUSHBUTTON_PIN        = 7  

  'Set the pin numbers for the step, direction, and disable pins
  'for the stepper driver.
  DIRECTION_PIN         = 3
  STEP_PIN              = 4  
  DISABLE_PIN           = 5

  'This setting is the starting and stopping speed of the
  'motor.  Please note that the RAMP will be subtracted from
  'this setting during the first loop.  Higher settings decrease
  'initial startup speed.  
  START_STOP_SPEED      = 50_000

  'This setting is the ramp incrementor/decrementor, and it will
  'affect just how fast the motor ramps up to top speed and
  'ramps down until the motor stops.  
  RAMP                  = 25

PUB Main

  'Set the direction of the pushbutton pin as an input.
  DIRA[PUSHBUTTON_PIN]~

  'Set the directions of the direction, step, and disable pins
  'for the stepper driver as outputs.
  DIRA[STEP_PIN]~~
  DIRA[DIRECTION_PIN]~~
  DIRA[DISABLE_PIN]~~

  'Enable the motor.
  OUTA[DISABLE_PIN]~~

  'Set the direction of rotation.
  OUTA[DIRECTION_PIN]~

  'Loop until there is input on the PUSHBUTTON_PIN and then stop the iteration.
  REPEAT
  
    IF INA[PUSHBUTTON_PIN]

      QUIT   

  'Start motor operation.  
  RunDaStepperMotor(STEP_PIN, START_STOP_SPEED, MAX_EXE_MOTOR_SPEED, RAMP)  

PUB RunDaStepperMotor(StepPin, CycleOffset, MaxExeMotorSpeed, RampIncDec) | Ramps, Counter

  'Set up the CTRMODE of Counter A for NCO/PWM single-ended.
  CTRA[30..26] := %00100

  'Set the output pin for Counter A.
  CTRA[5..0] := StepPin

  'Set the value to be added to PHSA with every clock cycle.
  FRQA := 1

  'Set APIN as an output.
  DIRA[StepPin]~~

  'Get the current System Counter value.
  Counter := CNT

  'Ramp up motor speed 
  REPEAT WHILE CycleOffset => MaxExeMotorSpeed

    'Send out a high pulse on the step pin for the desired duration.
    PHSA := -HIGH_PULSE_WIDTH

    'Wait for a specified period of time before send sending another
    'high pulse to the step pin.
    WAITCNT(Counter += CycleOffset -= RampIncDec)

    Ramps++

  'Please note that the pushbutton will be retained to start stepper
  'operation, however, the pushbutton used in the following loop will
  'be changed to a detent switch at a later point in time to permit a
  'more automated process.

  'Run the motor at full speed until we get contrary input and then
  'stop this iteration.
  REPEAT WHILE NOT INA[PUSHBUTTON_PIN]
 
    'Send out a high pulse on the step pin for the desired duration.
    PHSA := -HIGH_PULSE_WIDTH

    'Wait for a specified period of time before send sending another
    'high pulse to the step pin.
    WAITCNT(Counter += CycleOffset)

  'Ramp down the motor and come to a stop.
  REPEAT Ramps
 
    'Send out a high pulse on the step pin for the desired duration.
    PHSA := -HIGH_PULSE_WIDTH

    'Wait for a specified period of time before send sending another
    'high pulse to the step pin.
    WAITCNT(Counter += CycleOffset += RampIncDec)

  'Return to the Main function and wait for a button press to start
  'the process all over again.
  Main

  